# -*- encoding: utf-8 -*-
###########################################################################
#    Module Writen to OpenERP, Open Source Management Solution
#
#    Copyright (c) 2010 Tecnos Consultores S.C
#    All Rights Reserved.
############################################################################
#    Coded by: Fernando Irene Garcia (fernandoig1125@hotmail.com)
############################################################################
#
#    This program is free software: you can redistribute it and/or modify
#    it under the terms of the GNU Affero General Public License as
#    published by the Free Software Foundation, either version 3 of the
#    License, or (at your option) any later version.
#
#    This program is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU Affero General Public License for more details.
#
#    You should have received a copy of the GNU Affero General Public License
#    along with this program.  If not, see <http://www.gnu.org/licenses/>.
#
##############################################################################

import time
import base64
import tools
from crm import crm
from osv import fields
from osv import osv
from tools.translate import _

class calidad_calidad(osv.osv):
    _name='calidad.calidad'
    
    _columns = {
        'date_reception':fields.date('Fecha de Recepcion', required=True),
        'tanque':fields.char('Tanque', size=64, required=True),
        'temperatura':fields.float('Temperatura'),
        'acidez':fields.float('Acidez'),
        'grasa':fields.float('Grasa(%)'),
        'densidad':fields.float('Densidad gr/Lt'),
        'ph':fields.float('PH'),
        'solidos':fields.float('Solidos totales(%)'),
        'proteina':fields.float('Proteina(%)'),
        'antibiotico':fields.float('Antibiotico', size=64),
        'analista':fields.many2one('res.users', 'Analista'),
        'production_id':fields.many2one('mrp.production.calidad', 'Produccion'),
        'dictamen':fields.selection([('liberado','LIBERADO'), ('rechazado', 'RECHAZADO')], 'Dictamen', required=True),
    }
    _defaults = {
        'analista':lambda self, cr, uid, context: uid,
        'date_reception': lambda *a: time.strftime('%Y-%m-%d'),
    }
calidad_calidad()

class calidad_grasa_requerido(osv.osv):
    _name='calidad.grasa.requerido'
    
    def _compute_descremada(self, cr, uid, ids, field_name, arg, context={}):
        resultado = {}
        for i in self.browse(cr, uid, ids, arg):
            values = self.read(cr, uid, [i.id], ['grasa', 'grasa_requerida', 'volumen', 'leche'])
            D7 = values[0]['grasa']
            F7 = values[0]['grasa_requerida']
            E7 = values[0]['volumen']
            G7 = values[0]['leche']
            res = (((D7-F7)+(F7-G7))*E7)
            if res == 0:
                raise osv.except_osv(_('Invalid action !'), _('No se puede realizar una division entre cero.!'))
            res = (((D7-F7)/((D7-F7)+(F7-G7))*E7))
            resultado[i.id] = res
        return resultado
        
    def _compute_entera(self, cr, uid, ids, field_name, arg, context={}):
        resultado = {}
        for i in self.browse(cr, uid, ids, arg):
            values = self.read(cr, uid, [i.id], ['grasa', 'grasa_requerida', 'volumen', 'leche'])
            D7 = values[0]['grasa']
            F7 = values[0]['grasa_requerida']
            E7 = values[0]['volumen']
            G7 = values[0]['leche']
            res = (((D7-F7)+(F7-G7))*E7)
            if res == 0:
                raise osv.except_osv(_('Invalid action !'), _('No se puede realizar una division entre cero.!'))
            res = (((F7-G7)/((D7-F7)+(F7-G7))*E7))
            resultado[i.id] = res
        return resultado
    
    _columns = {
        'date':fields.date('Fecha'),
        'categ_id':fields.many2one('product.category', 'Tipo', domain="[('type','=','normal')]"),
        'grasa':fields.float('Grasa(%)', help='Leche cruda Entera', required=True),
        'grasa_requerida':fields.float('Grasa Requerida(%)', required=True),
        'volumen':fields.float('Volumen', help='Volumen de Leche a producir', required=True),
        'leche':fields.float('Leche P/Ajustar', required=True),
        'leche_descremada':fields.function(_compute_descremada, method = True, type = 'float', string = 'Leche Descremada(Lts)'),
        'leche_entera':fields.function(_compute_entera, method = True, type = 'float', string = 'Leche Entera(Lts)'),
        'production_id':fields.many2one('mrp.production.calidad', 'Produccion'),
        'state':fields.char('Estado', size=64),
    }
    
    _defaults = {
        'grasa':lambda *a: 1.00,
        'grasa_requerida':lambda *a: 1.00,
        'volumen':lambda *a: 1.00,
        'date': lambda *a: time.strftime('%Y-%m-%d'),
        'state': lambda *a: 'draft',
    }
    def button_dummy(self, cr, uid, ids, context=None):
        return True
        
calidad_grasa_requerido()

class calidad_analisis(osv.osv):
    _name='calidad.analisis'
    
    _columns = {
        'product_id':fields.many2one('product.product', 'Producto', required=True),
        'lote':fields.char('Lote', size=64, required=True),
        'date_reception':fields.date('Fecha Recepcion'),
        'anaquel':fields.char('Vida Anaquel', size=64),
        'dictamen':fields.selection([('liberado', 'LIBERADO'), ('rechazado', 'RECHAZADO'), ('cuarentena','CUARENTENA')], 'Dictamen', required=True),
        
        'date_analisis':fields.date('Fecha de Analisis'),
        'bx':fields.float('Bx'),
        'acidez':fields.float('Acidez'),
        'ph':fields.float('PH'),
        'grasa':fields.float('Grasa(%)'),
        'humedad':fields.float('Humedad(%)'),
        'viscosidad':fields.float('Viscosidad(cps)'),
        'apariencia':fields.char('Apariencia', size=64),
        'textura':fields.char('Textura', size=64),
        'color':fields.char('Color', size=64),
        'sabor':fields.char('Sabor', size=64),
        'fundido':fields.char('Fundido', size=64),
        'analista':fields.many2one('res.users','Analizo'),
        
        'date_microbiologia':fields.date('Fecha de Analisis'),
        'coliformes':fields.float('Coliformes(UFC/ml o g)'),
        'cbt':fields.float('CBT(UFC/ml o g)'),
        'hongos':fields.float('Hongos/Levaduras(UFC/ml o g)'),
        'analista_microbiologia':fields.many2one('res.users','Analizo'),
        'production_id':fields.many2one('mrp.production.calidad', 'Produccion'),
    }
    
    _defaults = {
        'analista':lambda self, cr, uid, context: uid,
        'analista_microbiologia':lambda self, cr, uid, context: uid,
        'date_reception': lambda *a: time.strftime('%Y-%m-%d'),
        'date_analisis': lambda *a: time.strftime('%Y-%m-%d'),
        'date_microbiologia': lambda *a: time.strftime('%Y-%m-%d'),
    }
calidad_analisis()
    
class calidad_monitoreo(osv.osv):
    _name='calidad.monitoreo'
    
    _columns = {
        'date':fields.date('Fecha', required=True),
        'area_muestra':fields.char('Area de Muestra', size=64, help="AREA DE TOMA DE MUESTRA", required=True),
        'concentracion':fields.float('Concentracion(0.5 - 1.5 p.p.m. Cl)'),
        'coliformes':fields.float('ColiformesF.(Metodo Colilert)'),
        'coliformes_total':fields.float('Coliformes Totales.(Metodo Colilert)'),
        'ph':fields.float('PH'),
        'analista':fields.many2one('res.users','Analizo'),
        'production_id':fields.many2one('mrp.production.calidad', 'Produccion'),
    }
    
    _defaults = {
        'analista':lambda self, cr, uid, context: uid,
        'date': lambda *a: time.strftime('%Y-%m-%d'),
    }
calidad_monitoreo()    
    
class calidad_verificacion(osv.osv):
    _name='calidad.verificacion'
    
    _columns = {
        'date_check':fields.date('Fecha de revision', required=True),
        'operadores_ids':fields.one2many('calidad.operador', 'verificacion_id', 'Operadores'),
    }
    
    _defaults = {
        'date_check': lambda *a: time.strftime('%Y-%m-%d'),
    }
calidad_verificacion()

class calidad_operador(osv.osv):
    _name='calidad.operador'
    
    def _calificaiones_get(self, cr, uid, context=None):
        calificaciones = [('4', 'Excelente'),('3','Bien'),('2','Regular'),('1','Mal')]
        return calificaciones
        
    _columns = {
        'operador_id':fields.many2one('res.users', 'Operador', required=True),
        'uniforme':fields.selection(_calificaiones_get, 'Uniforme'),
        'botas':fields.selection(_calificaiones_get, 'Botas'),
        'mandil':fields.selection(_calificaiones_get, 'Mandil'),
        'unas':fields.selection(_calificaiones_get, 'Unas'),
        'joyeria':fields.selection(_calificaiones_get, 'Joyeria'),
        'bano':fields.selection(_calificaiones_get, 'Bano'),
        'heridas':fields.selection(_calificaiones_get, 'Heridas'),
        'cofia':fields.selection(_calificaiones_get, 'Cofia'),
        'cubrebocas':fields.selection(_calificaiones_get, 'Cubrebocas'),
        'observaciones':fields.text('Observaciones'),
        'verificacion_id':fields.many2one('calidad.verificacion', 'Verificacion')
    }
    
calidad_operador()

class calidad_materia_prima(osv.osv):
    _name='calidad.materia.prima'
    
    _columns = {
        'date_reception':fields.date('Fecha de recepcion', required=True),
        'code_mp':fields.char('Codigo de MP y/o Empaque', size=64),
        'tipo_mp':fields.char('Tipo de MP y/o Empaque', size=64),
        'proveedor_id':fields.many2one('res.partner', 'Proveedor', domain="[('supplier','=',True)]"),
        'lote':fields.char('No.Lote', size=64, required=True),
        'date_due':fields.date('Fecha de Caducidad'),
        'cantidad':fields.integer('Cantidad recibida'),
        'parametro':fields.char('Parametro', size=64, help="En este punto se especificaria el parametro a medir para evaluar la materia prima y/o material de empaque."),
        'especificacion':fields.char('Especificacion', size=64, help="Seria la especificacin de cada materia prima o material de empaque."),
        'resultados':fields.text('Resultados'),
        'analista_fisicoquimico':fields.many2one('res.users', 'Analista Fisicoquimico'),
        'analista_microbiologia':fields.many2one('res.users', 'Analista Microbiologia'),
        'dictamen':fields.selection([('liberado', 'LIBERADO'),('rechazado','RECHAZADO')], 'Dictamen', required=True),
        'production_id':fields.many2one('mrp.production.calidad', 'Produccion'),
    }
    
    _defaults = {
        'analista_fisicoquimico':lambda self, cr, uid, context: uid,
        'analista_microbiologia':lambda self, cr, uid, context: uid,
        'date_reception': lambda *a: time.strftime('%Y-%m-%d'),
    }
calidad_materia_prima()

class calidad_maduracion_yoghurt(osv.osv):
    _name='calidad.maduracion.yoghurt'
    
    _columns = {
        'date':fields.date('Fecha', required=True),
        'cantidad_leche':fields.float('Cantidad de Leche'),
        'lote':fields.char('Lote', size=64, required=True),
        'cultivo':fields.char('Cultivo', size=64, help="Aqu se indica el cultivo que se utiliza para la inoculacion del yogurt."),
        'date_inoculacion':fields.datetime('Hora de Inoculacion'),
        'analista':fields.many2one('res.users', 'Analizo'),
        'dictamen':fields.selection([('liberado', 'LIBERADO'),('rechazado','RECHAZADO')], 'Dictamen', required=True),
        
        'acidez_cruda':fields.float('Acidez D'),
        'grasa_cruda':fields.float('Grasa(%)'),
        'ph_cruda':fields.float('PH'),
        'solidos_totales':fields.float('Solidos Totales'),
        'densidad':fields.float('Densidad'),
        
        'acidez_yoghurt':fields.float('Acidez D'),
        'grasa_yoghurt':fields.float('Grasa(%)'),
        'ph_yoghurt':fields.float('PH'),
        
        'acidez_corte':fields.float('Acidez D'),
        'date_corte':fields.datetime('Hora de corte de Yoghurt'),
        'ph_corte':fields.float('PH'),
        'production_id':fields.many2one('mrp.production.calidad', 'Produccion'),
    }
    
    _defaults = {
        'analista':lambda self, cr, uid, context: uid,
        'date': lambda *a: time.strftime('%Y-%m-%d'),
    }
calidad_maduracion_yoghurt()
    
class crm_claim(crm.crm_case, osv.osv):
    _inherit='crm.claim'
    
    _columns={
        'lote':fields.char('Lote', size=64),
        'cantidad':fields.integer('Cantidad'),
        'date_entrega':fields.date('Fecha de Entrega'),
        'date_due':fields.date('Fecha de Caducidad'),
    }
crm_claim()
    
    
    
    
    
    
    
    
    
    
    
    
    
    
    
    
    
    
    
    