# -*- encoding: utf-8 -*-
##############################################################################
#
#    OpenERP, Open Source Management Solution
#    Copyright (C) 2004-2009 Tiny SPRL (<http://tiny.be>). All Rights Reserved
#    $Id$
#
#    This program is free software: you can redistribute it and/or modify
#    it under the terms of the GNU General Public License as published by
#    the Free Software Foundation, either version 3 of the License, or
#    (at your option) any later version.
#
#    This program is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU General Public License for more details.
#
#    You should have received a copy of the GNU General Public License
#    along with this program.  If not, see <http://www.gnu.org/licenses/>.
#
##############################################################################

from osv import fields
from osv import osv
import ir
from mx.DateTime import *
import netsvc
import time
from tools.translate import _
import math
import locale
from datetime import datetime, timedelta
from dateutil.relativedelta import relativedelta
locale.setlocale(locale.LC_ALL, 'C')

class mrp_bom(osv.osv):
    _inherit = 'mrp.bom'
    
    _columns={
        'product_qty': fields.float('Product Qty', required=True,digits=(16, 3)),
        'secundario':fields.boolean('Secundario?'),
        'qty_kg': fields.float('Kgs Qty', required=True,digits=(16, 3)),
        'qty_standard': fields.float('Cantidad Standard'),
    }
mrp_bom()

class mrp_tinas(osv.osv):
    _name= 'mrp.tinas'
    
    _columns={
        'name':fields.char('Nombre', size=256, required=True),
        'capacidad':fields.integer('Capacidad', required=True),
        'descripcion':fields.char('Descripcion', size=256)
    }
mrp_tinas()

class mrp_production_product_line(osv.osv):
    _inherit = 'mrp.production.product.line'
    
    _columns={
        'product_qty': fields.float('Product Qty', required=True,digits=(16, 6)),
    }
mrp_production_product_line()

class mrp_sugerido(osv.osv):
    _name= 'mrp.sugerido'
    
    _columns={
        'name':fields.char('Nombre', size=256, required=True),
        'date':fields.datetime('Fecha de Creacion'),
        'date_from':fields.datetime('Fecha Incio de Produccion', required=True),
        'date_to':fields.datetime('Fecha Fin de Produccion', required=True),
        'product_ids':fields.one2many('mrp.sugerido.lines', 'sugerido_id', 'Lista de Productos'),
        'requerimiento_ids':fields.one2many('mrp.sugerido.requerimientos', 'sugerido_id', 'Requerimientos'),
        'agrupado_ids':fields.one2many('mrp.sugerido.agrupados', 'sugerido_id', 'Productos Agrupados'),
        'productions_ids':fields.one2many('mrp.sugerido.productions', 'sugerido_id', 'Producciones'),
        'derivados_ids':fields.one2many('mrp.production.derivados', 'sugerido_id', 'Despiece de Derivados'),
        'state': fields.selection([('draft','Draft'),('done', 'Done')], 'Estado', readonly=True),
    }
    
    _defaults = {
        'state': lambda *a: 'draft',
    }
    
    def compute_derivados(self, cr, uid, ids, context=None):
        derivados_obj = self.pool.get('mrp.production.derivados')
        requerimiento_obj = self.pool.get('mrp.sugerido.requerimientos')
        derivados_ids = self.read(cr, uid, ids, ['derivados_ids'])
        for derivado in derivados_obj.browse(cr, uid, derivados_ids[0]['derivados_ids']):
            if derivado.padre_ok == 1:
                padre_id = derivado.name.id
                childs_ids = requerimiento_obj.search(cr, uid, [('padre_id', '=', padre_id), ('sugerido_id', '=', ids[0])])
                piezas_padre = 0
                for child in requerimiento_obj.browse(cr, uid, childs_ids):
                    piezas_padre += child.piezas
                childs_derivados_ids = derivados_obj.search(cr, uid, [('padre_id', '=', derivado.id), ('sugerido_id', '=', ids[0])])
                piezas_derivados = 0
                for child_derivado in derivados_obj.browse(cr, uid, childs_derivados_ids):
                    piezas_derivados += child_derivado.piezas
                piezas_total = piezas_padre-piezas_derivados
                if piezas_total >= 0:
                    derivados_obj.write(cr, uid, [derivado.id], {'piezas':piezas_total})
            else:
                derivados_total_mrp = 0
                piezas_derivado = 0
                lines_ids = derivados_obj.search(cr, uid, [('padre_id', '=', derivado.padre_id.id), ('sugerido_id', '=', ids[0]), ('production_id', '=', derivado.production_id.id)])
                for line in derivados_obj.browse(cr, uid, lines_ids):
                    derivados_total_mrp += line.production_id.piezas_totales
                    piezas_derivado += line.piezas
                    piezas = line.piezas
                    presentacion = line.name.presentacion
                    kilos_mrp = line.production_id.kilos
                    qty_mrp = line.production_id.product_qty
                    litros = ((piezas*presentacion)*qty_mrp)/kilos_mrp
                    derivados_obj.write(cr, uid, [line.id], {'litros':litros})
                derivados_totales = (derivados_total_mrp/len(lines_ids))-piezas_derivado
                if derivados_totales < 0:
                    raise osv.except_osv(_('Error!!'), _('Has sobrepasado la cantidad de piezas de la produccion : %s \nNumero de piezas: %s'%(derivado.production_id.name, derivado.production_id.piezas_totales)))
                else:
                    derivados_obj.write(cr, uid, lines_ids, {'piezas_total_mrp':derivados_totales})
        return True
    
    def check_date_planned(self, cr, uid, ids, date_planned='', tina_id=0, sale_ok=True, date_from='', date_to=''):
        production_obj = self.pool.get('mrp.production')
        date_planned_res = date_planned
        contador = 0
        for i in range (0, 8):
            try:
                date_planned = date_planned_res + relativedelta(days = contador)
            except:
                date_planned = datetime.strptime(date_planned_res, '%Y-%m-%d %H:%M:%S') + relativedelta(days = contador)
            if date_planned > datetime.strptime(date_to, '%Y-%m-%d %H:%M:%S'):
                date_planned_res = datetime.strptime(date_from, '%Y-%m-%d %H:%M:%S')
                contador = 0
            else:
                tinas_ids = production_obj.search(cr, uid, [('date_planned', '=', (date_planned).strftime('%Y-%m-%d')), ('tina_id', '=', tina_id)])
                if len(tinas_ids) < 2:
                    return date_planned
                contador+=1
        return date_planned
        
    def create_productions(self, cr, uid, ids, context=None):
        productions=[]
        production_obj = self.pool.get('mrp.production')
        bom_obj = self.pool.get('mrp.bom')
        sugerido_agrupado_obj = self.pool.get('mrp.sugerido.agrupados')
        sugerido_requerimientos_obj = self.pool.get('mrp.sugerido.requerimientos')
        production_calidad_obj = self.pool.get('mrp.production.calidad')
        production_derivados_obj = self.pool.get('mrp.production.derivados')
        sugerido_production_obj = self.pool.get('mrp.sugerido.productions')
        lines = sugerido_agrupado_obj.search(cr, uid, [('sugerido_id', 'in', ids)])
        date_sugerido = self.browse(cr, uid, ids)[0].date_from
        for line in sugerido_agrupado_obj.browse(cr, uid, lines):
            piezas = 0
            products = sugerido_requerimientos_obj.search(cr, uid, [('padre_id', '=', line.padre_id.id), ('sugerido_id', '=', line.sugerido_id.id)])
            cantidad_leche = float(line.cantidad_leche)
            batch = float(line.batch)
            num_productions = round(cantidad_leche/batch, 2)
            decimal = str(num_productions).split('.')[1]
            if int(decimal) > 50:
                num_productions = math.ceil(num_productions)
            else:
                num_productions = math.floor(num_productions)
            for requerimiento in sugerido_requerimientos_obj.browse(cr, uid, products):
                piezas += requerimiento.piezas
            padre_derivado_id = production_derivados_obj.create(cr, uid, {
                                'name':line.padre_id.id,
                                'code':line.padre_id.default_code,
                                'sugerido_id':ids[0],
                                'padre_ok':True,
                                'piezas':piezas,
                                'batch':line.batch,})
            for i in range(0, int(num_productions)):
                derivados = []
                derivados_ids = []
                tina_id = line.tina_id.id
                sale_ok = line.sale_ok
                date_from = line.sugerido_id.date_from
                date_to = line.sugerido_id.date_to
                if tina_id == 0:
                    raise osv.except_osv(_('Error!!'), _('No hay una tina definida para este producto : %s'%line.padre_id.name))
                bom_id = production_obj.product_id_change(cr, uid, ids, line.padre_id.id, context=None)['value']['bom_id']
                values_bom = bom_obj.browse(cr, uid, [bom_id])[0]
                if sale_ok == 1:
                    date_planned = datetime.strptime(line.date, '%Y-%m-%d') - relativedelta(days = line.padre_id.dias_entrega)
                else:
                    date_planned = datetime.strptime(line.date, '%Y-%m-%d')
                if date_planned < datetime.strptime(date_sugerido, '%Y-%m-%d %H:%M:%S'):
                    date_planned = date_sugerido
                if bom_id == 0:
                    raise osv.except_osv(_('Error!!'), _('No se tiene definida una Lista de Materiales para este producto : %s'%line.padre_id.name))
                date_planned = self.check_date_planned(cr, uid, ids, date_planned, tina_id, sale_ok, date_from, date_to)
                production_id = production_obj.create(cr, uid, {
                        'product_id': line.padre_id.id,
                        'product_qty': line.batch, 
                        'product_uom': line.batch_uom_id.id,
                        'tina_id': tina_id,
                        'station_id': 23,
                        'location_dest_id': 31,
                        'location_src_id': 31,
                        'bom_id': bom_id, 
                        'sugerido_id': ids[0], 
                        'routing_id':values_bom.routing_id.id, 
                        'kilos':(values_bom.qty_kg*line.batch)/values_bom.product_qty,
                        'date_planned': date_planned.strftime('%Y-%m-%d'),
                        'udm_masa_id': line.cantidad_uom_id.id,
                    })
                for product in sugerido_requerimientos_obj.browse(cr, uid, products):
                    if product.product_id.id not in derivados:
                        product_id = product.product_id.id
                        code = product.name
                        derivado_id = production_derivados_obj.create(cr, uid, {
                                                        'name':product_id,
                                                        'code':code,
                                                        'sugerido_id':ids[0],
                                                        'production_id':production_id,
                                                        'padre_id':padre_derivado_id,
                                                        'date':date_planned.strftime('%Y-%m-%d'),
                                                        'batch':line.batch,
                                                        'tina_id':line.tina_id.id,
                                                        'kilos':(values_bom.qty_kg*line.batch)/values_bom.product_qty,})
                        derivados.append(product.product_id.id)
                        derivados_ids.append(derivado_id)
                #Calculando cantidad de piezas por lote de produccion
                mrp_derivados = production_obj.read(cr, uid, [production_id], ['derivados_ids'])
                ctd_derivados = len(mrp_derivados[0]['derivados_ids'])
                piezas_totales = 0
                for derivado in production_derivados_obj.browse(cr, uid, mrp_derivados[0]['derivados_ids']):
                    presentacion = derivado.name.presentacion
                    kilos_lote = (values_bom.qty_kg*line.batch)/values_bom.product_qty
                    kilos_asignados = kilos_lote/ctd_derivados
                    piezas_derivado = kilos_asignados/presentacion
                    piezas_totales += piezas_derivado
                production_derivados_obj.write(cr, uid, derivados_ids, {'piezas_total_mrp':piezas_totales})
                production_obj.write(cr, uid, [production_id], {'piezas_totales':piezas_totales})
                sugerido_production_id = sugerido_production_obj.create(cr, uid, {
                                                       'sugerido_id':ids[0],
                                                       'production_id':production_id,
                                                       'batch':line.batch,
                                                       'date':date_planned.strftime('%Y-%m-%d'),
                                                        })
                production_derivados_obj.write(cr, uid, derivados_ids, {'sugerido_production_id':sugerido_production_id})
                calidad_id = production_calidad_obj.create(cr, uid, {'name':production_id, 'folio':production_id})
                production_obj.write(cr, uid, [production_id], {'calidad_id':calidad_id})
                productions.append(production_id)
        production_obj.action_compute(cr, uid, productions, properties=[])
        self.write(cr, uid, ids, {'state':'done'})
        return {
               'name':_("Producciones"),
               'view_mode': 'tree,form,calendar,graph',
               'view_id': False,
               'view_type': 'form',
               'res_model': 'mrp.production',
               'type': 'ir.actions.act_window',
               'domain': [('id', 'in', productions)],
               'context': dict(context, active_ids=ids),
               'nodestroy': True 
               }
mrp_sugerido()

class mrp_sugerido_productions(osv.osv):
    _name = 'mrp.sugerido.productions'
    
    _columns = {
        'sugerido_id':fields.many2one('mrp.sugerido', 'Sugerido'),
        'production_id':fields.many2one('mrp.production', 'Produccion'),
        'batch':fields.integer('Batch'),
        'date':fields.date('Fecha Planeada'),
        'derivados_ids':fields.one2many('mrp.production.derivados', 'sugerido_production_id', 'Derivados')
    
    }
mrp_sugerido_productions()

class mrp_production_derivados(osv.osv):
    _name= 'mrp.production.derivados'
    
    def write(self, cr, uid, ids, vals={}, context=None):
        returned = super(mrp_production_derivados, self).write(cr, uid, ids, vals=vals, context=context)
        production_obj = self.pool.get('mrp.production')
        try:
            production_id = vals['production_id']
            production_obj.action_compute(cr, uid, [production_id], properties=[])
            return returned
        except:
            return returned
            
    def _get_child_ids(self, cr, uid, ids, field_name, arg, context=None):
        result = {}
        for record in self.browse(cr, uid, ids, context=context):
            result[record.id] = self.search(cr, uid, [('padre_id', '=', record.id)])
        return result
        
    _columns={
        'name':fields.many2one('product.product', 'Producto', required=True),
        'code':fields.char('Codigo', size=64, required=True),
        'kilos':fields.float('Cantidad Masa'),
        'kilos_uom_id':fields.many2one('product.uom', 'UdM Masa'),
        'kilos_real':fields.integer('Kilos Real'),
        'piezas':fields.integer('Piezas Planeadas'),
        'piezas_total_mrp':fields.integer('Piezas Total MRP'),
        'piezas_uom_id':fields.many2one('product.uom', 'UdM Piezas Planeadas'),
        'piezas_mrp_uom_id':fields.many2one('product.uom', 'UdM Piezas MRP'),
        'piezas_real':fields.integer('Piezas Real'),
        'litros':fields.integer('Litros Planeados'),
        'litros_real':fields.integer('Litros Real'),
        'production_id':fields.many2one('mrp.production', 'Produccion'),
        'sugerido_production_id':fields.many2one('mrp.sugerido.productions', 'Produccion de Sugerido'),
        'sugerido_id':fields.many2one('mrp.sugerido', 'Sugerido'),
        'padre_id':fields.many2one('mrp.production.derivados', 'Padre'),
        'padre_ok':fields.boolean('Padre?'),
        'date':fields.datetime('Fecha'),
        'batch':fields.integer('Batch'),
        'batch_uom_id':fields.many2one('product.uom', 'UdM Batch'),
        'tina_id':fields.many2one('mrp.tinas', 'Tina'),
        'child_id': fields.function(_get_child_ids, method=True, type='many2many', relation="mrp.sugerido.requerimientos", string="Child Products"),
    }
    
    _defaults = {
        'piezas_uom_id': lambda self, cr, uid, context: self.pool.get('product.uom').search(cr, uid, [('name', '=', 'Pieza')])[0],
        'piezas_mrp_uom_id': lambda self, cr, uid, context: self.pool.get('product.uom').search(cr, uid, [('name', '=', 'Pieza')])[0],
        'batch_uom_id': lambda self, cr, uid, context: self.pool.get('product.uom').search(cr, uid, [('name', '=', 'Lt')])[0],
        'kilos_uom_id': lambda self, cr, uid, context: self.pool.get('product.uom').search(cr, uid, [('name', '=', 'kg')])[0],
    }
mrp_production_derivados()

class mrp_history_line(osv.osv):
    _name = 'mrp.history.line'
    
    _columns={
        'name': fields.char('Motivo', required=True, size=128),
        'date_last':fields.datetime('Fecha Anterior', readonly=True),
        'date_current':fields.datetime('Fecha Nueva', required=False),
        'user_id':fields.many2one('res.users', 'Usuario quien modifico', readonly=True),
        'production_id':fields.many2one('mrp.production', 'Produccion')
    }
    _defaults = {
        'user_id':lambda self, cr, uid, context: uid,
    }
mrp_history_line()

class mrp_production(osv.osv):
    _inherit = 'mrp.production'
    
    def action_compute(self, cr, uid, ids, properties=[]):
        prod_line_obj = self.pool.get('mrp.production.product.line')
        res = super(mrp_production, self).action_compute(cr, uid, ids, properties=[])
        production=self.browse(cr,uid,ids)[0]
        for lin in production.derivados_ids:
            litro_presentacion=((lin.piezas*lin.name.presentacion)*production.product_qty)/production.kilos
            for lins in lin.name.bom_extra_ids:
                if lins.tipo=='empaque':
                    qty_extra=lins.cantidad*lin.piezas
                if lins.tipo=='mp':
                    qty_kg=lin.piezas*lin.name.presentacion
                    qty_extra=lins.cantidad*qty_kg
                prod_line_obj.create(cr,uid,{
                    'name': lins.product_id2.default_code,
                    'product_id': lins.product_id2.id,
                    'product_qty': qty_extra,
                    'product_uom': lins.product_id2.product_tmpl_id.uom_id.id,
                    'product_uos_qty': qty_extra,
                    'product_uos': production.product_uos and production.product_uos.id or False,
                    'production_id': ids[0]})
        return res
        
    def action_confirm(self, cr, uid, ids):
        stock_obj = self.pool.get('stock.picking')
        move_obj = self.pool.get('stock.move')
        seq_obj = self.pool.get('ir.sequence')
        wf_service = netsvc.LocalService("workflow")
        
        picking_id = super(mrp_production, self).action_confirm(cr, uid, ids)
        values = self.read(cr, uid, ids, ['station_id', 'tina_id', 'origin'])
        order_id = values[0]['id']
        if values[0]['origin']:
            production_id = self.search(cr, uid, [('name','=',values[0]['origin'][1:])])
            values = self.read(cr, uid, production_id, ['station_id', 'tina_id'])
            self.write(cr, uid, [order_id], {'station_id':values[0]['station_id'][0], 'tina_id':values[0]['tina_id'][0]})
            stock_obj.write(cr, uid, [picking_id], {'station_id':values[0]['station_id'][0], 'tina_id':values[0]['tina_id'][0]})
        else:
            stock_obj.write(cr, uid, [picking_id], {'station_id':values[0]['station_id'][0], 'tina_id':values[0]['tina_id'][0]})
        cr.execute(""" 
                DELETE FROM stock_move WHERE id IN(
                    SELECT sm.id FROM mrp_production_move_ids mp 
                        JOIN stock_move sm
                        ON sm.id=mp.move_id 
                        JOIN product_bom_extra pb
                        ON pb.product_id2=sm.product_id
                        WHERE mp.production_id=%s
                        AND pb.tipo='empaque')
            """,(ids[0],))
        production=self.browse(cr,uid,ids)[0]
        routing_loc = None
        pick_type = 'internal'
        address_id = False
        if production.bom_id.routing_id and production.bom_id.routing_id.location_id:
            routing_loc = production.bom_id.routing_id.location_id
            if routing_loc.usage <> 'internal':
                pick_type = 'out'
            address_id = routing_loc.address_id and routing_loc.address_id.id or False
            routing_loc = routing_loc.id
        pick_name = seq_obj.get(cr, uid, 'stock.picking.' + pick_type)
        new_picking = stock_obj.create(cr, uid, 
                            {
            'name': pick_name,
            'origin': (production.origin or '').split(':')[0] + ':' + production.name,
            'type': 'internal',
            'state': 'draft',
            'company_id': production.company_id.id,
            'production_id':ids[0]
                            })
        for lin in production.move_lines:
            move_obj.write(cr,uid,[lin.id],{'picking_id':new_picking,'state':'confirmed'})
        return picking_id
    
    def create_picks(self,cr,uid,ids,type):
        seq_obj = self.pool.get('ir.sequence')
        pick_obj = self.pool.get('stock.picking')
        move_obj = self.pool.get('stock.move')
        production=self.browse(cr,uid,ids)[0]
        
        routing_loc = None
        pick_type = 'internal'
        address_id = False
        if production.bom_id.routing_id and production.bom_id.routing_id.location_id:
            routing_loc = production.bom_id.routing_id.location_id
            if routing_loc.usage <> 'internal':
                pick_type = 'out'
            address_id = routing_loc.address_id and routing_loc.address_id.id or False
            routing_loc = routing_loc.id
        pick_name = seq_obj.get(cr, uid, 'stock.picking.' + pick_type)

        picking_id = pick_obj.create(cr, uid, {
            'name': pick_name,
            'origin': (production.origin or '').split(':')[0] + ':' + production.name,
            'type': pick_type,
            'state': 'draft',
            'company_id': production.company_id.id,
            'production_id':ids[0]
            })
            
        source = production.product_id.product_tmpl_id.property_stock_production.id
        for lin in production.derivados_ids:
            for lins in lin.name.bom_extra_ids:
                if lins.tipo==type:
                    if type=='mp':
                        qty_kg=lin.piezas*lin.name.presentacion
                        qty=lins.cantidad*qty_kg
                    if type=='empaque':
                        qty=lins.cantidad*lin.piezas_real
                    data = {
                        'name':'PROD:' + production.name,
                        'date': production.date_planned,
                        'product_id': lins.product_id2.id,
                        'product_qty': qty,
                        'product_uom': lins.product_id2.uom_id.id,
                        'product_uos_qty': production.product_uos and production.product_uos_qty or False,
                        'product_uos': production.product_uos and production.product_uos.id or False,
                        'location_id': source,
                        'location_dest_id': production.location_dest_id.id,
                        'move_dest_id': production.move_prod_id.id,
                        'state': 'waiting',
                        'company_id': production.company_id.id,
                        'picking_id':picking_id
                    }
                    mov_id = move_obj.create(cr, uid, data)
                    cr.execute(""" insert into mrp_production_move_ids values(%s,%s) """,(production.id,mov_id))
        return True
    
    def action_termina(self,cr,uid,ids,context={}):
        move_obj = self.pool.get('stock.move')
        production=self.browse(cr,uid,ids)[0]
        for move_id in ids:
            self.action_produce(cr, uid, move_id,
                                production.product_qty,'consume_produce', context=context)
            self.write(cr,uid,ids,{'state':'despiece'})
            id_mov_prod=move_obj.search(cr,uid,[('production_id','=',move_id),('product_id','=',production.product_id.id)])
            move_obj.write(cr,uid,id_mov_prod,{'product_qty':production.kilos,'product_uom':13})
        
        for lins in production.pick_ids:
            if lins.state not in ['done','cancel']:
                raise osv.except_osv(_(' Erro !'), _('Pickings Sin Realizar') )
#        self.create_picks(cr,uid,ids,'mp')
        self.write(cr,uid,ids,{'state':'in_despiece'})
        return True
    
    
    def write(self, cr, uid, ids, vals={}, context=None):
        user_obj = self.pool.get('res.users')
        change_ok = user_obj.browse(cr, uid, [uid])[0].change_date_ok
        if change_ok == 1:
            returned = super(mrp_production, self).write(cr, uid, ids, vals, context=context)
            return returned
        else:
            if ids:
                values = self.browse(cr, uid, ids, context=context)[0]
                date_from = values.sugerido_id.date_from
                date_to = values.sugerido_id.date_to
            values_user = user_obj.browse(cr, uid, [uid])[0]
            if 'date_planned' in vals:
                date_expected = vals['date_planned']
                state = values.state
                tina_id = values.tina_id.id
                if state == 'draft':
                    tinas_ids = self.search(cr, uid, [('date_planned', '=', date_expected), ('tina_id', '=', tina_id)])
                    if len(tinas_ids) < 2:
                        agrupado_obj = self.pool.get('mrp.sugerido.agrupados')
                        agrupado_id = agrupado_obj.search(cr, uid, [('padre_id', '=', values.product_id.id), ('sugerido_id', '=', values.sugerido_id.id)])
                        values_agrupados = agrupado_obj.browse(cr, uid, agrupado_id)[0]
                        date_sale = values_agrupados.date
                        dias_entrega = values.product_id.dias_entrega
                        sale_ok = values_agrupados.sale_ok
                        if datetime.strptime(date_expected, '%Y-%m-%d %H:%M:%S') < datetime.strptime(date_from, '%Y-%m-%d %H:%M:%S') or datetime.strptime(date_expected, '%Y-%m-%d %H:%M:%S') > datetime.strptime(date_to, '%Y-%m-%d %H:%M:%S')+relativedelta(hours=23):
                            raise osv.except_osv(_('Error en fecha %s!'%date_expected), _('No se pueden asignar producciones fuera de la semana del sugerido:\nSemana del %s al %s.')%(values.sugerido_id.date_from, values.sugerido_id.date_to))
                        else:
                            if sale_ok == 0:
                                returned = super(mrp_production, self).write(cr, uid, ids, vals, context=context)
                                return returned
                            else:
                                date_planned = datetime.strptime(date_sale, '%Y-%m-%d')-relativedelta(days=dias_entrega)
                                if datetime.strptime(date_expected, '%Y-%m-%d %H:%M:%S') <= date_planned+relativedelta(hours=23):
                                    returned = super(mrp_production, self).write(cr, uid, ids, vals, context=context)
                                    return returned
                                else:
                                    raise osv.except_osv(_('Error en fecha %s!'%date_expected), _('No se puede modificar la fecha de produccion\nFecha Planeada de Entrega: %s')%date_sale)
                    else:
                        raise osv.except_osv(_('Error!!'), _('No puedes asignar otra produccion en esta Feha: %s a esta Tina: %s')%(date_expected, values.tina_id.name))
                else:
                    raise osv.except_osv(_('Error!!'), _('No se puede modificar la fecha de producciones confirmadas'))
            returned = super(mrp_production, self).write(cr, uid, ids, vals, context=context)
            return returned
        
    def _get_derivados(self, cr, uid, ids, field_name="", arg=[], context={}):
          data = {}
          qty=0
          for lins in self.browse(cr, uid, ids, context=context):
            if lins.derivados_ids:
                for lin in lins.derivados_ids:
                    qty +=1
                    data[lins.id] = qty
            else:
                data[lins.id] = 0
          return data
    
    def action_empaques(self,cr,uid,ids,context={}):
        production=self.browse(cr,uid,ids)[0]
        for lins in production.pick_ids:
            if lins.state not in ['done','cancel']:
                raise osv.except_osv(_(' Erro !'), _('Pickings Sin Realizar') )
        self.write(cr,uid,ids,{'state':'done'})
        return True
        
    _columns={
        'station_id':fields.many2one('stock.location', 'Estacion', domain=[('station_ok','=',True)], required=True),
        'kilos': fields.float('Cantidad Masa',digits=(16,4)),
        'piezas_totales': fields.integer('Piezas Totales',),
        'tina_id':fields.many2one('mrp.tinas', 'Tina', required=True),
        'calidad_id':fields.many2one('mrp.production.calidad', 'Registro de calidad', readonly=True),
        'derivados_ids':fields.one2many('mrp.production.derivados', 'production_id', 'Derivados', readonly=True),
        'sugerido_id':fields.many2one('mrp.sugerido', 'Sugerido', readonly=True),
        'udm_masa_id':fields.many2one('product.uom', 'UdM Masa', readonly=True),
        'history_ids':fields.one2many('mrp.history.line', 'production_id', 'Historial de cambios'),
        
        'inicio_agitacion_leche':fields.datetime('Inicio de Agitacion', help='Ingrese la hora de inicio de agitacion de la Leche'),  
        'grasa':fields.float('Grasa'),
        'acidez':fields.float('Acidez'),
        'densidad':fields.float('Densidad'),
        'ph_agitacion':fields.float('PH'),
        'antibiotico':fields.char('Antibiotico', size=64),
        'temperatura_agitacion':fields.char('Temperatura', size=64),
        'grasa_en_crema':fields.char('Grasa en Crema', size=64),
        'responsable_agitacion_id':fields.many2one('res.users','Responsable'),
        
        'inicio_pasteurizacion':fields.datetime('Inicio de Pasteurizacion', help='Ingrese la hora de inicio de pasteurizacion'),
        'temperatura_pasteurizacion':fields.char('T.Celsius Pasteurizacion', size=64),
        'ph_cultivo':fields.float('PH antes de agregar cultivos', help='Ingrese el nivel de PH antes de agregar cultivos'),
        'acidez_cultivo':fields.float('Acidez'),
        'temperatura_leche':fields.char('T.Celsius de Leche', size=64),
        'hora_adicion':fields.datetime('Hora de Adicion', help='Ingrese la hora de adicion de cultivos'),  
        'adicion_calcio':fields.char('Adicion de Cloruro de calcio', size=64),
        'grasa_cuajar':fields.char('Grasa al Cuajar', size=64),
        'acidez_adicion':fields.float('Acidez'),
        'ph_adicion':fields.float('PH'),
        'fosfatasa':fields.float('Fosfatasa'),
        'hora_adicion_cuajo':fields.datetime('Hora de Adicion de Cuajo', help='Ingrese la hora de adicion de cuajo'),
        'temperatura_cuajado':fields.char('Temperatura de Cuajado', size=64),
        'responsable_adicion_id':fields.many2one('res.users','Responsable'),
        
        'hora_inversion_cuajada':fields.datetime('Hora Cuajada', help='Ingrese la hora de inversion de Cuajada'),
        'hora_corte_cuajada':fields.datetime('Hora Corte Cuajada', help='Ingrese la hora de corte de Cuajada'),
        'hora_reposo':fields.datetime('Hora Reposo'),
        'hora_reposo2':fields.datetime('Hora Reposo'),
        'ph_suero_corte':fields.float('PH de suero al corte'),
        'acidez_suero_corte':fields.float('Acidez de suero al corte'),
        'hora_inicio_agitacion':fields.datetime('Hora Inicio Agitacion'),
        'acidez_suero':fields.float('Acidez Suero'),
        'ph_suero':fields.float('PH Suero'),
        'hora_inicio_coccion':fields.datetime('Hora Inicio Coccion'),
        'temperatura_coccion':fields.char('Temperatura', size=64),
        'hora_inicio_agitacion2':fields.datetime('Hora Inicio Agitacion'),
        'acidez_suero2':fields.float('Acidez Suero'),
        'ph_suero2':fields.float('PH Suero'),
        'hora_inicio_prepensado':fields.datetime('Hora Inicio Preprensado'),
        'acidez_suero3':fields.float('Acidez Suero'),
        'ph_pasta':fields.float('PH de pasta'),
        'hora_inicio_moldeo':fields.datetime('Hora Inicio de Moldeo'),
        'ph_pasta2':fields.float('PH de pasta'),
        'hora_inicio_prensado':fields.datetime('Hora Inicio de Prensado'),
        'hora_volteo':fields.datetime('Hora Volteo'),
        'hora_segundo_prensado':fields.datetime('Hora de Segundo Prensado'),
        'ph_queso':fields.float('PH de queso'),
        'total':fields.char('Total', size=64),
        'piezas':fields.char('Piezas', size=64),
        'kilos2':fields.char('Kilos', size=64),
        'rendimiento':fields.char('Rendimiento', size=64),
        'responsable_proceso_id':fields.many2one('res.users','Responsable'),
        'state': fields.selection([('draft','Draft'),('picking_except', 'Picking Exception'),('confirmed','Waiting Goods'),('ready','Ready to Produce'),('in_production','In Production'),('cancel','Cancelled'),('done','Done'),('empaque','Esperando Material Empaque'),
                    ('despiece','Esperando Ingredientes'),('in_despiece','En Despiece'),('in_empaque','En empaque')],'State', readonly=True,
                    help='When the production order is created the state is set to \'Draft\'.\n If the order is confirmed the state is set to \'Waiting Goods\'.\n If any exceptions are there, the state is set to \'Picking Exeption\'.\
                    \nIf the stock is available then the state is set to \'Ready to Produce\'.\n When the production gets started then the state is set to \'In Production\'.\n When the production is over, the state is set to \'Done\'.'),
        'derivados': fields.function(_get_derivados, method=True, string='derivados', type='float' ),
        'pick_ids': fields.one2many('stock.picking', 'production_id', 'Pickings'),
    }
mrp_production()

class stock_picking(osv.osv):
    _inherit = 'stock.picking'
    _columns= {
        'production_id':fields.many2one('mrp.production', 'Produccion'),
    }
stock_picking()

class mrp_sugerido_lines(osv.osv):
    _name= 'mrp.sugerido.lines'
    
    _columns={
        'name':fields.char('Codigo', size=256, required=True),
        'product_id':fields.many2one('product.product', 'Producto', required=True),
        'padre_id':fields.many2one('product.product', 'Padre', required=True, domain=[('padre_ok', '=', True)]),
        'sugerido_id':fields.many2one('mrp.sugerido', 'Sugerido'),
        'dias_venta':fields.integer('Dias Venta'),
        'dias_cubrir':fields.integer('Dias por Cubrir'),
        'venta':fields.float('Venta'),
        'avg_venta':fields.float('Promedio Venta'),
        'saldo_stock':fields.float('Stock Saldo'),
        'stock_minimo':fields.float('Stock Minimo'),
        'cantidad_necesaria':fields.float('Cantidad Necesaria'),
        'cantidad_sugerida':fields.integer('Cantidad Sugerida'),
        'cantidad_sugerida_uom_id':fields.many2one('product.uom', 'UdM Cantidad Sugerida'),
        'pedido_stock':fields.float('Pedido por Stock'),
        'pedido_stock_minimo':fields.float('Pedido por Stock Minimo'),
        'date':fields.date('Fecha'),
        'sale_ok':fields.boolean('Venta?'),
    }
    
    def onchange_product_line(self, cr, uid, ids, product_id):
        product_obj = self.pool.get('product.product')
        product = product_obj.browse(cr, uid, [product_id])
        name = product[0].default_code
        padre = product[0].padre_id.id
        result = {'value': {'padre_id':padre, 'name':name}}
        return result
        
mrp_sugerido_lines()

class mrp_sugerido_requerimientos(osv.osv):
    _name= 'mrp.sugerido.requerimientos'
    
    _columns={
        'name':fields.char('Codigo', size=256, required=True),
        'product_id':fields.many2one('product.product', 'Producto', required=True, domain=[('padre_ok', '=', True)]),
        'padre_id':fields.many2one('product.product', 'Padre', required=True),
        'sugerido_id':fields.many2one('mrp.sugerido', 'Sugerido'),
        'presentacion':fields.float('Presentacion'),
        'kilos':fields.integer('Cantidad'),
        'kilos_uom_id':fields.many2one('product.uom', 'UdM Cantidad'),
        'piezas':fields.float('Unidades'),
        'piezas_uom_id':fields.many2one('product.uom', 'UdM Unidades'),
        'date':fields.date('Fecha'),
        'sale_ok':fields.boolean('Venta'),
    }
    
    _defaults = {
        'kilos_uom_id': lambda self, cr, uid, context: self.pool.get('product.uom').search(cr, uid, [('name', '=', 'kg')])[0],
    }
mrp_sugerido_requerimientos()

class mrp_sugerido_agrupados(osv.osv):
    _name= 'mrp.sugerido.agrupados'
    
    _columns={
        'padre_id':fields.many2one('product.product', 'Producto', required=True, domain=[('padre_ok', '=', True)]),
        'sugerido_id':fields.many2one('mrp.sugerido', 'Sugerido'),
        'cantidad':fields.float('Cantidad'),
        'cantidad_uom_id':fields.many2one('product.uom', 'UdM Cantidad'),
        'rendimiento':fields.integer('Rendimiento(%)'),
        'cantidad_leche':fields.integer('Cantidad Leche'),
        'leche_uom_id':fields.many2one('product.uom', 'UdM Leche'),
        'batch':fields.integer('Batch'),
        'batch_uom_id':fields.many2one('product.uom', 'UdM Batch'),
        'date':fields.date('Fecha'),
        'sale_ok':fields.boolean('Venta?'),
        'tina_id':fields.many2one('mrp.tinas', 'Tina'),
    }
    
    _defaults = {
        'leche_uom_id': lambda self, cr, uid, context: self.pool.get('product.uom').search(cr, uid, [('name', '=', 'Lt')])[0],
        'batch_uom_id': lambda self, cr, uid, context: self.pool.get('product.uom').search(cr, uid, [('name', '=', 'Lt')])[0],
        'cantidad_uom_id': lambda self, cr, uid, context: self.pool.get('product.uom').search(cr, uid, [('name', '=', 'kg')])[0],
    }
mrp_sugerido_agrupados()

class explosiona(osv.osv):
    _name= 'explosiona'
    
    _columns={
        'name':fields.char('Folio',size=256),
        'productos':fields.char('Productos',size=512),
        'lineas_ids':fields.one2many('explosiona.line','explo_id','Lineas'),
        
    }
explosiona()

class explosiona_line(osv.osv):
    _name='explosiona.line'
    
    def get_product_available(self, cr, uid, ids, context=None):
        produ=self.pool.get('product.product')
        if context is None:
            context = {}
        states = context.get('states',[])
        what = context.get('what',())
        if not ids:
            ids = self.search(cr, uid, [])
        res = {}.fromkeys(ids, 0.0)
        if not ids:
            return res

        if context.get('shop', False):
            cr.execute('select warehouse_id from sale_shop where id=%s', (int(context['shop']),))
            res2 = cr.fetchone()
            if res2:
                context['warehouse'] = res2[0]

        if context.get('warehouse', False):
            cr.execute('select lot_stock_id from stock_warehouse where id=%s', (int(context['warehouse']),))
            res2 = cr.fetchone()
            if res2:
                context['location'] = res2[0]

        if context.get('location', False):
            if type(context['location']) == type(1):
                location_ids = [context['location']]
            else:
                location_ids = context['location']
        else:
            location_ids = []
            wids = self.pool.get('stock.warehouse').search(cr, uid, [], context=context)
            for w in self.pool.get('stock.warehouse').browse(cr, uid, wids, context=context):
                location_ids.append(w.lot_stock_id.id)

        # build the list of ids of children of the location given by id
        if context.get('compute_child',True):
            child_location_ids = self.pool.get('stock.location').search(cr, uid, [('location_id', 'child_of', location_ids)])
            location_ids = child_location_ids or location_ids
        else:
            location_ids = location_ids

        states_str = ','.join(map(lambda s: "'%s'" % s, states))

        uoms_o = {}
        product2uom = {}
        for product in produ.browse(cr, uid, ids, context=context):
            product2uom[product.id] = product.uom_id.id
            uoms_o[product.uom_id.id] = product.uom_id

        prod_ids_str = ','.join(map(str, ids))
        location_ids_str = ','.join(map(str, location_ids))
        results = []
        results2 = []

        from_date=context.get('from_date',False)
        to_date=context.get('to_date',False)
        date_str = ''
        date_args = ()
        if from_date and to_date:
            date_str = "and date_planned>=%s and date_planned<=%s"
            date_args = (from_date, to_date)
        elif from_date:
            date_str = "and date_planned>=%s"
            date_args = (from_date,)
        elif to_date:
            date_str = "and date_planned<=%s"
            date_args = (to_date,)

        if 'in' in what:
            # all moves from a location out of the set to a location in the set
            cr.execute(
                'select sum(product_qty), product_id, product_uom '\
                'from stock_move '\
                'where location_id not in %s '\
                'and location_dest_id in %s '\
                'and product_id in %s '\
                'and state in %s '+ date_str + ' '\
                'group by product_id,product_uom',
                (tuple(location_ids), tuple(location_ids), tuple(ids),
                 tuple(states)) + date_args)
            results = cr.fetchall()
        if 'out' in what:
            # all moves from a location in the set to a location out of the set
            cr.execute(
                'select sum(product_qty), product_id, product_uom '\
                'from stock_move '\
                'where location_id in %s '\
                'and location_dest_id not in %s '\
                'and product_id in %s '\
                'and state in %s '+ date_str + ' '\
                'group by product_id,product_uom',
                (tuple(location_ids), tuple(location_ids), tuple(ids),
                 tuple(states)) + date_args)
            results2 = cr.fetchall()
        uom_obj = self.pool.get('product.uom')
        uoms = map(lambda x: x[2], results) + map(lambda x: x[2], results2)
        if context.get('uom', False):
            uoms += [context['uom']]

        uoms = filter(lambda x: x not in uoms_o.keys(), uoms)
        if uoms:
            uoms = uom_obj.browse(cr, uid, list(set(uoms)), context=context)
        for o in uoms:
            uoms_o[o.id] = o
        for amount, prod_id, prod_uom in results:
            amount = uom_obj._compute_qty_obj(cr, uid, uoms_o[prod_uom], amount,
                    uoms_o[context.get('uom', False) or product2uom[prod_id]])
            res[prod_id] += amount
        for amount, prod_id, prod_uom in results2:
            amount = uom_obj._compute_qty_obj(cr, uid, uoms_o[prod_uom], amount,
                    uoms_o[context.get('uom', False) or product2uom[prod_id]])
            res[prod_id] -= amount
        return res

    def _product_available(self, cr, uid, ids, field_names=None, arg=False, context={}):
        ids2=[]
        res = {}
        for id in ids:
            le=self.browse(cr,uid,[id],context={})[0]
            ids2.append(le.product_id.id)
        c = context.copy()
        c.update({ 'states':('confirmed','waiting','assigned','done'), 'what':('in', 'out') })
        stock=self.get_product_available(cr,uid,ids2,context=c)
        for id in ids:
            rr=self.browse(cr,uid,[id],context={})[0]
            res[id] = stock.get(rr.product_id.id, 0.0)
        return res
    
    def _product_available_r(self, cr, uid, ids, field_names=None, arg=False, context={}):
        ids2=[]
        res = {}
        for id in ids:
            le=self.browse(cr,uid,[id],context={})[0]
            ids2.append(le.product_id.id)
        c = context.copy()
        c.update({ 'states':('done',), 'what':('in', 'out') })
        stock=self.get_product_available(cr,uid,ids2,context=c)
        for id in ids:
            rr=self.browse(cr,uid,[id],context={})[0]
            res[id] = stock.get(rr.product_id.id, 0.0)
        return res
    
    _columns={
        'product_id':fields.many2one('product.product','Producto'),
        'product_uom':fields.many2one('product.uom', 'Unidad de Medida'),
        'product_qty':fields.float('Cantidad'),
        'qty_available': fields.function(_product_available_r, method=True, type='float', string='Real Stock', ),
        'virtual_available': fields.function(_product_available, method=True, type='float', string='Virtual Stock'),
        'explo_id':fields.many2one('explosiona','Lineas'),
        
    }
    
explosiona_line()
