# -*- coding: utf-8 -*-
##############################################################################
#
#    OpenERP, Open Source Management Solution
#    Copyright (C) 2004-2010 Tiny SPRL (<http://tiny.be>).
#
#    This program is free software: you can redistribute it and/or modify
#    it under the terms of the GNU Affero General Public License as
#    published by the Free Software Foundation, either version 3 of the
#    License, or (at your option) any later version.
#
#    This program is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU Affero General Public License for more details.
#
#    You should have received a copy of the GNU Affero General Public License
#    along with this program.  If not, see <http://www.gnu.org/licenses/>.
#
##############################################################################
from osv import fields, osv
from tools.translate import _


class mrp_presentaciones(osv.osv_memory):
     _name='mrp.presentaciones'
     
     def view_init(self, cr, uid, fields_list, context=None):
        """
         Creates view dynamically and adding fields at runtime.
         @param self: The object pointer.
         @param cr: A database cursor
         @param uid: ID of the user currently logged in
         @param context: A standard dictionary
         @return: New arch of view with new columns.
        """
        if context is None:
            context = {}
        res = super(mrp_presentaciones, self).view_init(cr, uid, fields_list, context=context)
        record_id = context and context.get('active_id', False)
        pick_obj = self.pool.get('mrp.production')
        pick = pick_obj.browse(cr, uid, record_id, context=context)
        for m in [line for line in pick.derivados_ids]:
              if 'return%s'%(m.name.id) not in self._columns:
                  self._columns['return%s'%(m.name.id)] = fields.float(string=m.name.name, required=True)
        return res

     def fields_view_get(self, cr, uid, view_id=None, view_type='form',
                        context=None, toolbar=False, submenu=False):
        """
         Changes the view dynamically
         @param self: The object pointer.
         @param cr: A database cursorSemana de produccion del 02 al 07 de enero del 2012.
         @param uid: ID of the user currently logged in
         @param context: A standard dictionary
         @return: New arch of view.
        """
        res = super(mrp_presentaciones, self).fields_view_get(cr, uid, view_id=view_id, view_type=view_type, context=context, toolbar=toolbar,submenu=False)
        record_id = context and context.get('active_id', False)
        active_model = context.get('active_model')
        if  active_model != 'mrp.production':
            return res
        if record_id:
            pick_obj = self.pool.get('mrp.production')
            pick = pick_obj.browse(cr, uid, record_id)
            return_history = {}
            res['fields'].clear()
            arch_lst=['<?xml version="1.0"?>', '<form string="%s">' % _('Cantidad Producto'), '<label string="%s" colspan="4"/>' % _('Cantidad Presentaciones.')]
            for m in pick.derivados_ids:
                return_history[m.name.id] = 0
                arch_lst.append('<field name="return%s"/>\n<newline/>' % (m.name.id))
                res['fields']['return%s' % m.name.id]={'string':m.name.name, 'type':'float', 'required':True}
                res.setdefault('returns', []).append(m.name.id)
            arch_lst.append('<group col="2" colspan="4">')
            arch_lst.append('<button icon="gtk-cancel" special="cancel" string="Cancel" />')
            arch_lst.append('<button name="create_presentaciones" string="Lineas" colspan="1" type="object" icon="gtk-apply" />')
            arch_lst.append('</group>')
            arch_lst.append('</form>')
            res['arch'] = '\n'.join(arch_lst)
        return res
    
    
     def create_presentaciones(self,cr,uid,ids,context={}):
          if context is None:
            context = {}
          prod_obj = self.pool.get('mrp.production')
          derivados_obj = self.pool.get('mrp.production.derivados')
          prod_der = self.pool.get('stock.move')
          pick_obj = self.pool.get('stock.picking')
          move_ids = context.get('active_ids', [])
          
          partial_datas={}

          qty_validar=0
          data=self.read(cr,uid,ids[0])
          production=prod_obj.browse(cr,uid,move_ids)[0]
          source = production.product_id.product_tmpl_id.property_stock_production.id

          for lin in production.derivados_ids:
               qty=data['return%s' % lin.name.id]
               derivados_obj.write(cr,uid,[lin.id],{'piezas_real':qty})
               qty_validar+=(qty*lin.name.presentacion)
               litro_presentacion=((qty*lin.name.presentacion)*production.product_qty)/production.kilos
               prod_der.create(cr,uid,{'name':'PROD:' + production.name,
                     'date': production.date_planned,
                     'product_id': lin.name.id,
                     'product_qty': qty,
                     'product_uom': lin.name.product_tmpl_id.uom_id.id,
                     'product_uos_qty': qty,
                     'product_uos': production.product_uos and production.product_uos.id or False,
                     'location_id': source,
                     'location_dest_id': production.location_dest_id.id,
                     'move_dest_id': production.move_prod_id.id,
                     'state': 'done',
                     'company_id': production.company_id.id,
                     'production_id':production.id})
          prod_obj.create_picks(cr,uid,move_ids,'empaque')
          prod_obj.write(cr,uid,move_ids,{'state':'empaque'})
          return {}
mrp_presentaciones()

class mrp_termina(osv.osv_memory):
     _name='mrp.termina'
     def action_termina(self,cr,uid,ids,context={}):
#          move_ids = context.get('active_ids', [])
 #         prod_obj = self.pool.get('mrp.production')
  #        production=prod_obj.browse(cr,uid,move_ids)[0]
   #       prod_obj.write(cr,uid,[production.id],{'state':'empaque'})
    #      return{}
    
        if context is None:
            context = {}
        prod_obj = self.pool.get('mrp.production')
        move_ids = context.get('active_ids', [])
        production=prod_obj.browse(cr,uid,move_ids)[0]
        for data in self.read(cr, uid, ids, context=context):
            for move_id in move_ids:
                prod_obj.action_produce(cr, uid, move_id,
                                    production.product_qty, data['mode'], context=context)
        return {}
mrp_termina()

class mrp_empaque(osv.osv_memory):
     _name='mrp.empaque'
    
     def default_get(self, cr, uid, fields, context=None):
        """
         To get default values for the object.
         @param self: The object pointer.
         @param cr: A database cursor
         @param uid: ID of the user currently logged in
         @param fields: List of fields for which we want default values
         @param context: A standard dictionary
         @return: A dictionary with default values for all field in ``fields``
        """
        if context is None:
            context = {}
        res = super(mrp_empaque, self).default_get(cr, uid, fields, context=context)
        record_id = context and context.get('active_id', False) or False
        mrp_obj = self.pool.get('mrp.production')
        prod = mrp_obj.browse(cr, uid, record_id, context=context)
        if prod:
            for line in prod.product_lines:
                return_id = 'return%s'%(line.product_id.id)
                if return_id in fields:
                    res[return_id] = line.product_qty
        return res
    
     def view_init(self, cr, uid, fields_list, context=None):
        """
         Creates view dynamically and adding fields at runtime.
         @param self: The object pointer.
         @param cr: A database cursor
         @param uid: ID of the user currently logged in
         @param context: A standard dictionary
         @return: New arch of view with new columns.
        """
        if context is None:
            context = {}
        res = super(mrp_empaque, self).view_init(cr, uid, fields_list, context=context)
        record_id = context and context.get('active_id', False)
        pick_obj = self.pool.get('mrp.production')
        pick = pick_obj.browse(cr, uid, record_id, context=context)
        for m in [line for line in pick.derivados_ids]:
          for e in m.name.bom_extra_ids:
            if e.tipo=='empaque':
              if 'return%s'%(e.product_id2.id) not in self._columns:
                  self._columns['return%s'%(e.product_id2.id)] = fields.float(string=m.name.name, required=True)
        return res

     def fields_view_get(self, cr, uid, view_id=None, view_type='form',
                        context=None, toolbar=False, submenu=False):
        """
         Changes the view dynamically
         @param self: The object pointer.
         @param cr: A database cursor
         @param uid: ID of the user currently logged in
         @param context: A standard dictionary
         @return: New arch of view.
        """
        res = super(mrp_empaque, self).fields_view_get(cr, uid, view_id=view_id, view_type=view_type, context=context, toolbar=toolbar,submenu=False)
        record_id = context and context.get('active_id', False)
        active_model = context.get('active_model')
        if  active_model != 'mrp.production':
            return res
        if record_id:
            pick_obj = self.pool.get('mrp.production')
            pick = pick_obj.browse(cr, uid, record_id)
            return_history = {}
            res['fields'].clear()
            arch_lst=['<?xml version="1.0"?>', '<form string="%s">' % _('Cantidad Producto'), '<label string="%s" colspan="4"/>' % _('Cantidad Presentaciones.')]
            for m in pick.derivados_ids:
                for e in m.name.bom_extra_ids:
                    if e.tipo=='empaque':
                        return_history[e.product_id2.id] = 0
                        arch_lst.append('<field name="return%s"/>\n<newline/>' % (e.product_id2.id))
                        res['fields']['return%s' % e.product_id2.id]={'string':e.product_id2.name, 'type':'float', 'required':True}
                        res.setdefault('returns', []).append(e.product_id2.id)
            arch_lst.append('<group col="2" colspan="4">')
            arch_lst.append('<button icon="gtk-cancel" special="cancel" string="Cancel" />')
            arch_lst.append('<button name="create_empaque" string="Lineas" colspan="1" type="object" icon="gtk-apply" />')
            arch_lst.append('</group>')
            arch_lst.append('</form>')
            res['arch'] = '\n'.join(arch_lst)
        return res
    
     def create_empaque(self,cr,uid,ids,context={}):
          if context is None:
            context = {}
          prod_obj = self.pool.get('mrp.production')
          prod_der = self.pool.get('stock.move')
          pick_obj = self.pool.get('stock.picking')
          move_ids = context.get('active_ids', [])
          
          partial_datas={}

          qty_validar=0
          data=self.read(cr,uid,ids[0])
          production=prod_obj.browse(cr,uid,move_ids)[0]
          source = production.product_id.product_tmpl_id.property_stock_production.id

          id_pick=pick_obj.search(cr,uid,[('production_id','=',move_ids[0]),('state','<>','done')])
        
          for pick in pick_obj.browse(cr,uid,id_pick):
              for move in pick.move_lines:
                    for lin in production.derivados_ids:
                        for lins in lin.name.bom_extra_ids:
                            if lins.product_id2.id==move.product_id.id:
                                qty2=data['return%s' % lins.product_id2.id]
#                                litro_presentacion2=((qty2*lin.name.presentacion)*lins.cantidad)/production.kilos
                                litro_presentacion2=qty2*lins.cantidad
                                partial_datas['move%s' % (move.id)] = {
                                    'product_id': move.product_id.id,
                                    'product_qty': litro_presentacion2,
                                    'product_uom': move.product_uom.id,
                                    'prodlot_id': move.prodlot_id.id,
                                }
              res=pick_obj.do_partial(cr, uid, [pick.id], partial_datas, context=context)
              if pick.id==res[pick.id]['delivered_picking']:
                prod_obj.write(cr,uid,move_ids,{'state':'done'})
          return {}

mrp_empaque()


class mrp_adicionales(osv.osv_memory):
     _name='mrp.adicionales'
    
     def view_init(self, cr, uid, fields_list, context=None):
        """
         Creates view dynamically and adding fields at runtime.
         @param self: The object pointer.
         @param cr: A database cursor
         @param uid: ID of the user currently logged in
         @param context: A standard dictionary
         @return: New arch of view with new columns.
        """
        if context is None:
            context = {}
        res = super(mrp_adicionales, self).view_init(cr, uid, fields_list, context=context)
        record_id = context and context.get('active_id', False)
        pick_obj = self.pool.get('mrp.production')
        pick = pick_obj.browse(cr, uid, record_id, context=context)
        for m in [line for line in pick.product_lines]:
          if 'return%s'%(m.product_id.id) not in self._columns:
              self._columns['return%s'%(m.product_id.id)] = fields.float(string=m.product_id.name, required=True)
        return res

     def fields_view_get(self, cr, uid, view_id=None, view_type='form',
                        context=None, toolbar=False, submenu=False):
        """
         Changes the view dynamically
         @param self: The object pointer.
         @param cr: A database cursor
         @param uid: ID of the user currently logged in
         @param context: A standard dictionary
         @return: New arch of view.
        """
        res = super(mrp_adicionales, self).fields_view_get(cr, uid, view_id=view_id, view_type=view_type, context=context, toolbar=toolbar,submenu=False)
        record_id = context and context.get('active_id', False)
        active_model = context.get('active_model')
        if  active_model != 'mrp.production':
            return res
        if record_id:
            pick_obj = self.pool.get('mrp.production')
            pick = pick_obj.browse(cr, uid, record_id)
            return_history = {}
            res['fields'].clear()
            arch_lst=['<?xml version="1.0"?>', '<form string="%s">' % _('Cantidad Producto'), '<label string="%s" colspan="4"/>' % _('Cantidad Presentaciones.')]
            for m in pick.product_lines:
                return_history[m.product_id.id] = 0
                arch_lst.append('<field name="return%s"/>\n<newline/>' % (m.product_id.id))
                res['fields']['return%s' % m.product_id.id]={'string':m.product_id.name, 'type':'float', 'required':True}
                res.setdefault('returns', []).append(m.product_id.id)
            arch_lst.append('<group col="2" colspan="4">')
            arch_lst.append('<button icon="gtk-cancel" special="cancel" string="Cancel" />')
            arch_lst.append('<button name="create_adicionales" string="Lineas" colspan="1" type="object" icon="gtk-apply" />')
            arch_lst.append('</group>')
            arch_lst.append('</form>')
            res['arch'] = '\n'.join(arch_lst)
        return res
    
     def create_adicionales(self,cr,uid,ids,context={}):
          if context is None:
            context = {}
          seq_obj = self.pool.get('ir.sequence')
          prod_obj = self.pool.get('mrp.production')
          prod_der = self.pool.get('stock.move')
          pick_obj = self.pool.get('stock.picking')
          move_ids = context.get('active_ids', [])

          qty_validar=0
          data=self.read(cr,uid,ids[0])
        
          production=prod_obj.browse(cr,uid,move_ids)[0]
          source = production.product_id.product_tmpl_id.property_stock_production.id
        
          pick_name = seq_obj.get(cr, uid, 'stock.picking.internal')
          picking_id = pick_obj.create(cr, uid, {
            'name': 'Producto Adicional' + pick_name,
            'origin': (production.origin or '').split(':')[0] + ':' + production.name,
            'type': 'internal',
            'state': 'draft',
            'company_id': production.company_id.id,
            'production_id':production.id
            })
            
          for pick in prod_obj.browse(cr,uid,move_ids):
              for move in pick.product_lines:
                  qty=data['return%s' % move.product_id.id]
                  if qty:
                      datas = {
                            'name':'PROD:' + production.name,
                            'date': production.date_planned,
                            'product_id': move.product_id.id,
                            'product_qty': qty,
                            'product_uom': move.product_id.uom_id.id,
                            'product_uos_qty': production.product_uos and production.product_uos_qty or False,
                            'product_uos': production.product_uos and production.product_uos.id or False,
                            'location_id': source,
                            'location_dest_id': production.location_dest_id.id,
                            'move_dest_id': production.move_prod_id.id,
                            'state': 'waiting',
                            'company_id': production.company_id.id,
                            'picking_id':picking_id
                        }
                      mov_id = prod_der.create(cr, uid, datas)
          return {}
mrp_adicionales()






